<?php
/**
 *  Kartra Amember integration plugin v1.0
 *  Copyright (c) Kartra
 *  All Rights Reserved
 **/

class Am_Paysystem_Kartra extends Am_Paysystem_Abstract
{

    const PLUGIN_STATUS = self::STATUS_PRODUCTION;
    const PLUGIN_REVISION = '1.0';

    protected $defaultTitle = "Kartra";
    protected $defaultDescription = "";

    public function _initSetupForm(Am_Form_Setup $form)
    {
        $form->addText("secret", array('size' => 40))
            ->setLabel(array("Secret Key", ''));
    }

    public function __construct(Am_Di $di, array $config)
    {

        parent::__construct($di, $config);

        foreach ($di->paysystemList->getList() as $k => $p)
        {
            if ($p->getId() == $this->getId())
                $p->setPublic(false);
        }
        $di->productTable->customFields()->add(
            new Am_CustomFieldText(
                'kartra_product_id',
                "Kartra connection ID",
                "")
        );
    }

    public function _process(Invoice $invoice, Am_Mvc_Request $request, Am_Paysystem_Result $result)
    {
        throw new Am_Exception_InputError("Not supported for this payment method!");
    }

    public function createTransaction(Am_Mvc_Request $request, Am_Mvc_Response $response, array $invokeArgs)
    {
        return new Am_Paysystem_Transaction_Kartra($this, $request, $response, $invokeArgs);
    }

    public function getRecurringType()
    {
        return self::REPORTS_REBILL;
    }

    public function canAutoCreate()
    {
        return true;
    }

    function getConfig($key = null, $default = null)
    {
        switch ($key)
        {
            case 'testing' : return false;
            case 'auto_create' : return true;
            default: return parent::getConfig($key, $default);
        }
    }

    function getReadme(){
        $url = $this->getPluginUrl('ipn');
        return <<<CUT
<b>Kartra integration</b>
1. In this page, input any alpha-numeric Secret Key of your choice.
2. In Amember -> Manage Products, create a product for each access level you want to offer to your customers. Example: Silver, Gold and Platinum.
3. In Kartra -> Sales -> Integrations, create as many individual membership integrations as different access levels you want to offer to your customers. Example: one for Silver, one for Gold and anothe$
4. While you are creating those integrations in Kartra, you will notice that you have to paste the same Secret Key you created in this page (the same Secret Key must be inserted in both platforms). Als$
5. We will now link each of those membership integrations you have created in Kartra with each of the products you have created here in Amember. Remember that you need to link ONE membership integratio$
6. Make sure the user welcome email is configured at Amember -> configuration -> Email -> Signup Email, so your new members receive their Amember login username and password.
7. From now on, every time you grant/cancel access to a membership from Kartra (because of a customer places an order, or because a lead triggers an automation, etc), the corresponding entry will be au$

CUT;

    }

    protected function _afterInitSetupForm(Am_Form_Setup $form)
    {
        parent::_afterInitSetupForm($form);
        $form->removeElementByName($this->_configPrefix . $this->getId() . '.auto_create');
    }


}

class Am_Paysystem_Transaction_Kartra extends Am_Paysystem_Transaction_Incoming
{

    /**
     * @var Invoice $invoice;
     */
    public $invoice;
    protected $_autoCreateMap = array(
        'name_f' => 'buyer_first_name',
        'name_l' => 'buyer_last_name',
        'email' => 'buyer_email',
        'user_external_id' => 'buyer_email',
        'invoice_external_id' => array('transaction_parent_id', 'transaction_id'),
    );

    public function getUniqId()
    {
        return $this->request->get('transaction_id');
    }

    public function validateSource()
    {
        $hash = $this->request->get('security_hash');

        $k = md5($s = $this->request->getInt('transaction_id') . $this->request->get('transaction_amount') . $this->getPlugin()->getConfig('secret')) == $hash;
        return $k;
    }

    public function generateInvoiceExternalId()
    {
        list($l,) = explode('-',$this->getUniqId());
        return $l;
    }
    public function getReceiptId()
    {
        return $this->request->get('transaction_id');
    }

    public function validateStatus()
    {
        return true;
    }

    public function validateTerms()
    {
        return true;
    }

    public function autoCreateGetProducts()
    {
        $connection_key = $this->request->get('connection_key');
        if (empty($connection_key)) {
            return;
        }

        $product = $this->getPlugin()->getDi()->productTable->findFirstByData('kartra_product_id', $connection_key);

        return $product;
    }

    function processValidated()
    {
        switch ($this->request->get('transaction_type'))
        {
            case 'sale' :
                $this->invoice->addPayment($this);
                break;
            case 'refund' :
                $this->invoice->addRefund($this, $this->request->get('transaction_parent_id'));
                break;
            case 'cancellation' :
                $this->invoice->setCancelled(true);
                break;
        }
        // Nothing to do here
    }

    function findInvoiceId()
    {
        return null;
    }

}
